#ifndef CimagenBNH
#define CimagenBNH
#include <extctrls.hpp>
#include "algoritmillos.h"
#include <math.h>
#include <math.hpp>
#include "matrices.h"

class CimagenBN//la representacion de las imagenes es matricial, no euclideana
   {
      public:
         double **canal;//arreglo de tres imagenes, que corresponden a los tres colores primarios

         //coeficientes de color es el vector de coeficientes para transformar una imagen de color a blanco y negro
         //usando una suma ponderada de sus componentes de colores primarios.
         double *coefColor;
         TImage *I;//nos sirve para no lidiar con broncas de lectura, escritura y exhibicin de imgenes
         int largo, alto;//dimensiones de la imagen
         double min, max;
         //*************Detalles de memoria e inicializacion***************************
         CimagenBN();
         CimagenBN(AnsiString archivo);
         CimagenBN(int l, int a);
         ~CimagenBN();
         void asignarCoef(double pR, double pV, double pA);
         void pedirMemoria(int l, int a);//asigna suficiente memoria a los arreglos para una imagen de l*a pixeles
         void liberarMemoria(void);
         //****************************************************************************
         //********************operadores binarios con asignacion**********************
         CimagenBN& operator +=(CimagenBN &IC);//suma IC a esta imagen
         CimagenBN& operator +=(CimagenBN *IC);//suma IC a esta imagen
         CimagenBN& operator +=(double x);//suma el escalar elemento a elemento
         CimagenBN& operator -=(CimagenBN &IC);//suma IC a esta imagen
         CimagenBN& operator -=(CimagenBN *IC);//suma IC a esta imagen
         CimagenBN& operator -=(double x);//resta el escalar elemento a elemento
         CimagenBN& operator *=(CimagenBN &IC);//multiplica IC por esta imagen
         CimagenBN& operator *=(CimagenBN *IC);//multiplica IC por esta imagen
         CimagenBN& operator *=(double x);//multiplica por el escalar elemento a elemento
         CimagenBN& operator /=(CimagenBN &IC);//divide elemento a elemento
         CimagenBN& operator /=(CimagenBN *IC);//divide elemento a elemento
         CimagenBN& operator /=(double x);//divide por el escalar elemento a elemento
         //******************************************************************
         //***********operadores binarios sin asignacion*************
         CimagenBN* operator +(CimagenBN &IC);//suma IC a esta imagen
         CimagenBN* operator +(CimagenBN *IC);//suma IC a esta imagen
         CimagenBN* operator +(double x);//suma el escalar elemento a elemento
         CimagenBN* operator -(CimagenBN &IC);//suma IC a esta imagen
         CimagenBN* operator -(CimagenBN *IC);//suma IC a esta imagen
         CimagenBN* operator -(double x);//resta el escalar elemento a elemento
         CimagenBN* operator *(CimagenBN &IC);//multiplica IC por esta imagen
         CimagenBN* operator *(CimagenBN *IC);//multiplica IC por esta imagen
         CimagenBN* operator *(double x);//multiplica por el escalar elemento a elemento
         CimagenBN* operator /(CimagenBN &IC);//divide elemento a elemento
         CimagenBN* operator /(CimagenBN *IC);//divide elemento a elemento
         CimagenBN* operator /(double x);//divide por el escalar elemento a elemento
         //******************************************************************
         //***********************asignaciones*******************************
         CimagenBN& operator =(CimagenBN &IC);
         CimagenBN& operator =(CimagenBN *IC);
         //******************************************************************

         //**************************entrada y salida************************
         void cargarImagen(AnsiString archivo);
         void guardarImagen(AnsiString archivo);
         void mostrarImagen(void);
         //******************************************************************

         //**********************proceso de imgenes*************************
         void cuantizar(int n, double *cortes, double *valores);//cuantizacion no uniforme
         /*aplica la funcion escalon(n) definida por cortes y valores, esto es f(x)=valores[j] siempre que
         x pertenezca a [ cortes[j], cortes[j+1] )  si j+1<n  o bien x pertenezca a [cortes[n-1], infinito)*/
         void cuantizar(int n);//cuantizacion uniforme
         void calcularHistograma(int n, int **cont);
         void ecualizar(int n, int *h);//ecualiza esta imagen usando el histograma h de n tonos
         void aplicarFuncion(double (*f)(double));
         void aplicarFuncion(double (*f)(double, double));
         void ceros(void);
         void normalizar(void);
         CimagenBN* conRuidoG(double m, double s);
         double media(void);//devuelve la media, la variable aleatoria es la intensidad
         double varianza(void);//devuelve la varianza, la variable aleatoria es la intensidad
         double minimo(void);
         double maximo(void);
         double suma(void);
         AnsiString estadisticas();
         CimagenBN* suavizaGaussiana(double sigma);
         CimagenBN* convolucionNoCircular(CimagenBN& IC);
         CimagenBN* convolucionSeparable(CimagenBN& K1, CimagenBN& K2);
         CimagenBN* derivadaX(int dir);
         CimagenBN* derivadaY(int dir);
         CimagenBN* derivadaGaussianaX(double sigma);
         CimagenBN* derivadaGaussianaY(double sigma);
         CimagenBN* derivadaGaussianaXY(double sigma);
         CimagenBN* derivadaGaussiana2X(double sigma);
         CimagenBN* derivadaGaussiana2Y(double sigma);
         CimagenBN* derivadaGaussiana3X(double sigma);
         CimagenBN* derivadaGaussiana3Y(double sigma);
         CimagenBN* laplacianoGaussiano(double sigma);
         CimagenBN* magnitudGradienteG(double sigma);
         CimagenBN* umbralPorcentual(double u);
         CimagenBN* umbral(double u);
         CimagenBN* umbralInvertido(double u);
         CimagenBN* umbralFijoInvertido(double u);
         CimagenBN* crucesPorCero(void);
         CimagenBN* detectarBordes(double sigma, double u);
         CimagenBN* detectorDireccional(double sigma, double u);         
         CimagenBN* derivadaDireccionalMG(double sigma, double u);
         CimagenBN* segundaDerivadaDireccionalMG(double sigma, double u);
         CimagenBN* derivadaGaussiana2XY(double sigma);
         CimagenBN* derivadaGaussiana2YX(double sigma);
         
         CimagenBN* correlacionCruzada(CimagenBN* h);//h es el kernel
         CimagenBN* dilatacion(CimagenBN* h);//h=elemento estructural
         CimagenBN* dilatacionGeneral(CimagenBN* h);
         CimagenBN* erosion(CimagenBN* ee);//ee="elemento estructural"
         CimagenBN* clasificar(void);
         void recuperarLinea(void);
         CimagenBN* gradienteResortes(CimagenBN *obs, double lambda, double (*)(double));
         CimagenBN* gradientePlacas(CimagenBN *obs, double lambda);
         double sumaPotencialVecinos(int f, int c, double (*)(double));
         int numVecinos(int f, int c);
         double magnitud(void);
         

   };

//************************Detalles de memoria e inicializacion***************************

CimagenBN::CimagenBN()
   {
      pedirMemoria(0,0);
      asignarCoef(1,1,1);
   }

CimagenBN::CimagenBN(AnsiString archivo)
   {
      pedirMemoria(0,0);      
      cargarImagen(archivo);
   }
CimagenBN::CimagenBN(int l, int a)
   {
      double r=1.0/3.0;
      pedirMemoria(l, a);
      asignarCoef(r,r,r);
      ceros();
   }

CimagenBN::~CimagenBN()
   {
      liberarMemoria();
   }

void CimagenBN::asignarCoef(double pR, double pV, double pA)
   {
      coefColor[0]=pR;
      coefColor[1]=pV;
      coefColor[2]=pA;
   }

void CimagenBN::pedirMemoria(int l, int a)
   {
      coefColor=(double*)malloc(3*sizeof(double));
      asignarCoef(1,1,1);
      max=-INFINITO;
      min=INFINITO;
      if (l==0 || a==0)//imagen vacia
         {
            largo=alto=0;
            canal=NULL;
            return;
         }
      largo=l;
      alto=a;
      canal=(double**)malloc(alto*sizeof(double*));
      for(int i=0;i<alto;i++)
         canal[i]=(double*)malloc(largo*sizeof(double));
   }

void CimagenBN::liberarMemoria(void)
   {
      for (int i=0;i<alto;i++)
         free(canal[i]);
      free(canal);
      free(coefColor);
   }
//****************************************************************************************
//*************************operadores binarios con asignacion*****************************

CimagenBN& CimagenBN::operator +=(CimagenBN &IC)
   {
      if (largo!=IC.largo || alto!=IC.alto)
         return *this;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]+=IC.canal[i][j];
      return *this;
   }

CimagenBN& CimagenBN::operator +=(CimagenBN *IC)
   {
      (*this)+=(*IC);
      delete IC;
      return *this;
   }
   
CimagenBN& CimagenBN::operator +=(double x)
   {
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]+=x;
      return *this;
   }   
   
CimagenBN& CimagenBN::operator -=(CimagenBN &IC)
   {
      if (largo!=IC.largo || alto!=IC.alto)
         return *this;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]-=IC.canal[i][j];
      return *this;
   }

CimagenBN& CimagenBN::operator -=(CimagenBN *IC)
   {
      (*this)-=(*IC);
      delete IC;
      return *this;
   }

CimagenBN& CimagenBN::operator -=(double x)
   {
      (*this)+=(-x);
      return *this;
   }   
   
CimagenBN& CimagenBN::operator *=(CimagenBN &IC)
   {
      if (largo!=IC.largo || alto!=IC.alto)
         return *this;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]*=IC.canal[i][j];
      return *this;
   }
   
CimagenBN& CimagenBN::operator *=(CimagenBN *IC)
   {
      (*this)*=(*IC);
      delete IC;
      return *this;
   }  
   
CimagenBN& CimagenBN::operator *=(double x)//multiplicacion escalar
   {
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]*=x;
      return *this;
   }
   
CimagenBN& CimagenBN::operator /=(CimagenBN &IC)
   {
      if (largo!=IC.largo || alto!=IC.alto)
         return *this;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            if(IC.canal[i][j] != 0)
               canal[i][j]/=IC.canal[i][j];
            else
               canal[i][j]=0;
      return *this;
   }
   
CimagenBN& CimagenBN::operator /=(CimagenBN *IC)
   {
      (*this)/=(*IC);
      delete IC;
      return *this;
   }  
   
CimagenBN& CimagenBN::operator /=(double x)//multiplicacion escalar
   {
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]*=x;
      return *this;
   }   

//******************************operadores binarios sin asignacion*************************************
   
CimagenBN* CimagenBN::operator +(CimagenBN &IC)//suma dos imagenes
   {
      if (largo!=IC.largo || alto!=IC.alto)
         return NULL;
      CimagenBN *aux;
      aux=new CimagenBN(largo,alto);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=canal[i][j]+IC.canal[i][j];
      return aux;
   }

CimagenBN* CimagenBN::operator +(CimagenBN *IC)//suma dos imagenes
  {
     CimagenBN *aux;
     aux=(*this)+(*IC);
     delete IC;
     return aux;
  } 

CimagenBN* CimagenBN::operator +(double x)//suma x a cada elemento de la imagen
   {
      CimagenBN *aux;
      aux=new CimagenBN(largo,alto);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=canal[i][j]+x;
      return aux;
      
   }

CimagenBN* CimagenBN::operator -(CimagenBN &IC)//suma dos imagenes
   {
      if (largo!=IC.largo || alto!=IC.alto)
         return NULL;
      CimagenBN *aux;
      aux=new CimagenBN(largo,alto);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=canal[i][j]-IC.canal[i][j];
      return aux;
   }

CimagenBN* CimagenBN::operator -(CimagenBN *IC)//suma dos imagenes
  {
     CimagenBN *aux;
     aux=(*this)-(*IC);
     delete IC;
     return aux;
  } 

CimagenBN* CimagenBN::operator -(double x)//resta x a cada elemento de la imagen
   {
      CimagenBN *aux;
      aux=new CimagenBN(largo,alto);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=canal[i][j]-x;
      return aux;

   }

CimagenBN* CimagenBN::operator *(CimagenBN &IC)//multiplica dos imagenes
   {
      if (largo!=IC.largo || alto!=IC.alto)
         return NULL;
      CimagenBN *aux;
      aux=new CimagenBN(largo,alto);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=canal[i][j]*IC.canal[i][j];
      return aux;
   }
   
CimagenBN* CimagenBN::operator *(CimagenBN *IC)//suma dos imagenes
  {
     CimagenBN *aux;
     aux=(*this)*(*IC);
     delete IC;
     return aux;
  }       

CimagenBN* CimagenBN::operator *(double x)//multiplica por x a cada elemento de la imagen
   {
      CimagenBN *aux;
      aux=new CimagenBN(largo,alto);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=canal[i][j]*x;
      return aux;
      
   }

CimagenBN* CimagenBN::operator /(CimagenBN &IC)//divide dos imagenes
   {
      if (largo!=IC.largo || alto!=IC.alto)
         return NULL;
      CimagenBN *aux;
      aux=new CimagenBN(largo,alto);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            if (IC.canal[i][j]==0)
               aux->canal[i][j]=0;
            else
               aux->canal[i][j]=canal[i][j]/IC.canal[i][j];
      return aux;
   }
   
CimagenBN* CimagenBN::operator /(CimagenBN *IC)//suma dos imagenes
  {
     CimagenBN *aux;
     aux=(*this)/(*IC);
     delete IC;
     return aux;
  }       
   
CimagenBN* CimagenBN::operator /(double x)//multiplica por x a cada elemento de la imagen
   {
      if(x==0)
         return NULL;
      CimagenBN *aux;
      aux=new CimagenBN(largo,alto);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=canal[i][j]/x;
      return aux;
   }

//********************************asignaciones********************************************

CimagenBN& CimagenBN::operator =(CimagenBN &IC)
   {
      if(largo!=IC.largo || alto!=IC.alto)
         return *this;
      for (int i=0;i<3;i++)
         coefColor[i]=IC.coefColor[i];
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]=IC.canal[i][j];
      return *this;
   }

CimagenBN& CimagenBN::operator =(CimagenBN *IC)
   {
      if(largo!=IC->largo || alto!=IC->alto)
         return *this;
      (*this)=(*IC);
      delete IC;
      return *this;

   }
//****************************************************************************************
//**********************************entrada y salida**************************************

void CimagenBN::ceros(void)
   {
      max=0;
      min=0;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]=0;
   }

void CimagenBN::cargarImagen(AnsiString archivo)
   {
      liberarMemoria();
      I->AutoSize=true;
      I->Picture->LoadFromFile(archivo);
      largo=I->Width;
      alto=I->Height;
      pedirMemoria(largo, alto);
      double aux;
      for(int i=0;i<alto;i++)
         {
            for (int j=0;j<largo;j++)
               {
                  aux=(((int)(I->Canvas->Pixels[j][i]))&filtroRojo)          *coefColor[0]+
                              ((((int)(I->Canvas->Pixels[j][i]))/tonoVerde)&filtroRojo)*coefColor[1]+
                              ((((int)(I->Canvas->Pixels[j][i]))/tonoAzul)&filtroRojo) *coefColor[2];
                  canal[i][j]=aux;
               }//for interno
         }//for externo
   }//cargarImagen

void CimagenBN::guardarImagen(AnsiString archivo)
   {
      mostrarImagen();
      I->Picture->SaveToFile(archivo);
   }

void CimagenBN::mostrarImagen(void)
   {
      
      max=-INFINITO;
      min=INFINITO;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            {
               if(canal[i][j]<min)
                  min=canal[i][j];
               if(canal[i][j]>max)
                  max=canal[i][j];
            }
      double rangoDinamico=max-min;
      int nivelGris;
      if (rangoDinamico==0)
         rangoDinamico=1;//no importa, solo se necesita algo distinto de cero
      I->Canvas->FillRect(I->Canvas->ClipRect);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            {
               nivelGris=((canal[i][j]-min)/rangoDinamico)*255.0;
               I->Canvas->Pixels[j][i]=(TColor)(tonoGris*nivelGris);
            }
   }
//****************************************************************************************
//******************************proceso de imgenes***************************************

void CimagenBN::normalizar(void)
   {
         double valMinimo=minimo(), valMaximo=maximo(), diferencia;
         diferencia=valMaximo-valMinimo;
         if(valMinimo==valMaximo)
            return;
         for (int i=0;i<alto;i++)
            for (int j=0;j<largo;j++)
               canal[i][j]=(canal[i][j]-valMinimo)/diferencia;
   }


CimagenBN* kernelGaussianoF(double sigma)
//genera un kernel gaussiano (fila) suficientemente grande dependiendo de sigma
   {
      int n=(int)(4.0*sigma);
      CimagenBN *aux=new CimagenBN(2*n+1,1);
      double suma=0, s2=2.0*sigma*sigma;

      aux->canal[0][n]=1.0;
      for (int i=1;i<=n;i++)
         suma+=(aux->canal[0][n+i]=exp(-(double)(i*i)/s2)) ;
        
      s2=1.0+2.0*suma;//la suma de todos los valores...
      aux->canal[0][n]/=s2;
      for(int i=1;i<=n;i++)
         aux->canal[0][n-i]=(aux->canal[0][n+i]/=s2) ;
      return aux;
   }

CimagenBN* kernelGaussianoC(double sigma)
//genera un kernel gaussiano (columna) suficientemente grande dependiendo de sigma
   {
      int n=(int)(4.0*sigma);
      CimagenBN *aux=new CimagenBN(1,2*n+1);
      double suma=0, s2=2.0*sigma*sigma;
      
      aux->canal[n][0]=1.0;
      for (int i=1;i<=n;i++)
         suma+=(aux->canal[n+i][0]=exp(-(double)(i*i)/s2)) ;
        
      s2=1.0+2.0*suma;//la suma de todos los valores...
      aux->canal[n][0]/=s2;
      for(int i=1;i<=n;i++)
         aux->canal[n-i][0]=(aux->canal[n+i][0]/=s2) ;
      return aux;
   }

CimagenBN* kernelDerivadaGX(double sigma)//kernel de derivadaGaussiana respecto a X
   {
      CimagenBN *aux;
      aux=kernelGaussianoF(sigma);
      int n=(aux->largo/2);
      float s2=1.0/(sigma*sigma);
      for (int i = 0 ; i <= 2*n ; i++)
         aux->canal[0][i]*=-s2*(i-n);
      return aux;
   }

CimagenBN* kernelDerivadaGY(double sigma)//kernel de derivadaGaussiana respecto a Y
   {
      CimagenBN *aux;
      aux=kernelGaussianoC(sigma);
      int n=(aux->alto/2);
      float s2=1.0/(sigma*sigma);
      for (int i = 0 ; i <= 2*n ; i++)
         aux->canal[i][0]*=-s2*(i-n);
      return aux;
   }

CimagenBN* kernelDerivada2GX(double sigma)//kernel de derivadaGaussiana respecto a X
   {
      CimagenBN *aux;
      
      aux=kernelGaussianoF(sigma);
      int n=(aux->largo/2);
      double s2=1.0/(sigma*sigma);
      for (int i = 0 ; i <= 2*n ; i++)
         aux->canal[0][i]*=s2*(s2*(i-n)*(i-n)-1);
      return aux;


   }

CimagenBN* kernelDerivada2GY(double sigma)//kernel de derivadaGaussiana respecto a Y
   {
      CimagenBN *aux;
      aux=kernelGaussianoC(sigma);
      int n=(aux->alto/2);
      double s2=1.0/(sigma*sigma);
      for (int i = 0 ; i <= 2*n ; i++)
         aux->canal[i][0]*=s2*(s2*(i-n)*(i-n)-1);
      return aux;
   }

CimagenBN* kernelDerivada3GX(double sigma)
   {
      CimagenBN *aux;
      aux=kernelGaussianoF(sigma);
      int n=(aux->largo/2);
      double s2=(sigma*sigma);
      double s6=1.0/(s2*s2*s2);
      
      for (int i = 0 ; i <= 2*n ; i++)
         aux->canal[0][i]*=s6*(s2*(i-n)-(i-n)*(i-n)*(i-n));
      return aux;   
   }

CimagenBN* kernelDerivada3GY(double sigma)
   {
      CimagenBN *aux;
      aux=kernelGaussianoC(sigma);
      int n=(aux->alto/2);
      double s2=(sigma*sigma);
      double s6=1.0/(s2*s2*s2);
      
      for (int i = 0 ; i <= 2*n ; i++)
         aux->canal[i][0]*=s6*(s2*(i-n)-(i-n)*(i-n)*(i-n));
      return aux;   
   }

CimagenBN* kernelGaussiano2D(double sigma)
//genera un kernel gaussiano suficientemente grande dependiendo de sigma
   {
      int n=(int)(4.0*sigma);
      CimagenBN *aux=new CimagenBN(2*n+1,2*n+1);      
      double suma=0, s2=2*sigma*sigma;      
      
      aux->canal[0][0]=1;
      for (int i=0;i<=n;i++)
         for (int j=0;j<=n;j++)
            {
               suma+=(aux->canal[n+i][n+j]=exp(-(float)(i*i+j*j)/s2));
            }
        
      s2=1.0+4.0*suma;//la suma de todos los valores...
      aux->canal[0][0]/=s2;
      for(int i=0;i<=n;i++)
         for (int j=0;j<=n;j++)
         {
            aux->canal[n-i][n+j]=(aux->canal[n+i][n+j]/=s2);;
            aux->canal[n+i][n-j]=aux->canal[n+i][n+j];
            aux->canal[n-i][n-j]=aux->canal[n+i][n+j];
         }
      return aux;
   }

double gaussiana(double s, double el, double th, double x, double y)//esta funcion es lenta... pero se entiende, es didactica
//s=desv. standard, el=elipticidad, th=angulo de inlinacion en radianes, (x,y)=punto de evaluacion
   {
      CMatriz R(2,2), Rt(2,2), D(2,2), T(2,2);
      Cvector v(2);
      v.M[0]=x;
      v.M[1]=y;
      D.cero();
      D.M[0][0]=1;
      D.M[1][1]=1.0/(el*el);
      R.M[0][0]=cos(th);    R.M[0][1]=sin(th)  ;
      R.M[1][0]=-R.M[0][1]; R.M[1][1]=R.M[0][0];
      Rt=R.transpuesta();
      T=R*(D*Rt);
      s=2*s*s;//varianza
      return exp(-((v*T)*v)/s) / (M_PI*s);
      
   }


CimagenBN* kernelGaussianoRot(double s, double el, double th)

   {
      int largo,alto;
      double x,y;
      largo=alto=(int)(4*el*s+1);
      x=largo/2.0;
      y=-alto/2.0;
      CimagenBN *aux= new CimagenBN(largo, alto);
      
      double **pixeles=aux->canal;
      double temp;
      
      CMatriz R(2,2), Rt(2,2), D(2,2), T(2,2);
      Cvector v(2);
      
      D.cero();
      D.M[0][0]=1;
      D.M[1][1]=1.0/(el*el);
      R.M[0][0]=cos(th);    R.M[0][1]=-sin(th)  ;
      R.M[1][0]=-R.M[0][1]; R.M[1][1]=R.M[0][0];
      Rt=R.transpuesta();
      T=R*(D*Rt);
      s=2*s*s;//varianza
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            {
               v.M[0]=j-x;
               v.M[1]=-i-y;               
               temp=exp(-((v*T)*v)/s) / (M_PI*s);
               pixeles[i][j]=temp;
            }
      return aux;
   }



CimagenBN* kernelDerivadaGX2D(double sigma)
//devuelve un kernel robusto de derivacin (respecto a X) en dos dimensiones, con parmetro sigma
   {
      int n=(int)(4.0*sigma);
      double factor=1.0/(sigma*sigma);
      CimagenBN *aux;
      aux=kernelGaussiano2D(sigma);            
      aux->canal[0][0]=0;
      for (int i=0;i<=n;i++)
         for (int j=-n;j<=n;j++)
            {
               aux->canal[n+i][n+j]=-factor*j*aux->canal[n+i][n+j];
               aux->canal[n-i][n+j]=aux->canal[n+i][n+j];
            }
      return aux;
   }

CimagenBN* kernelDerivadaGY2D(double sigma)
//devuelve un kernel robusto de derivacin (respecto a Y) en dos dimensiones, con parmetro sigma
   {
      int n=(int)(4.0*sigma);
      double factor=1.0/(sigma*sigma);
      CimagenBN *aux;
      aux=kernelGaussiano2D(sigma);            
      aux->canal[0][0]=0;
      for (int i=-n;i<=n;i++)
         for (int j=0;j<=n;j++)
            {
               aux->canal[n+i][n+j]=-factor*i*aux->canal[n+i][n+j];
               aux->canal[n+i][n-j]=aux->canal[n+i][n+j];
            }
      return aux;
   }

CimagenBN* kernelDerivada2GX2D(double sigma)
//devuelve un kernel robusto de derivacin (2 veces respecto a X), con parmetro sigma
   {
      int n=(int)(4.0*sigma);
      double factor=1.0/(sigma*sigma);
      CimagenBN *aux;
      aux=kernelGaussiano2D(sigma);            
      aux->canal[0][0]=0;
      for (int i=0;i<=n;i++)
         for (int j=-n;j<=n;j++)
            {
               aux->canal[n+i][n+j]=factor*(factor*j*j-1)*aux->canal[n+i][n+j];
               aux->canal[n-i][n+j]=aux->canal[n+i][n+j];
            }
      return aux;
   }

CimagenBN* kernelDerivada2GY2D(double sigma)
//devuelve un kernel robusto de derivacin (2 veces respecto a Y), con parmetro sigma
   {
      int n=(int)(4.0*sigma);
      double factor=1.0/(sigma*sigma);
      CimagenBN *aux;
      aux=kernelGaussiano2D(sigma);            
      aux->canal[0][0]=0;
      for (int i=-n;i<=n;i++)
         for (int j=0;j<=n;j++)
            {
               aux->canal[n+i][n+j]=factor*(factor*i*i-1)*aux->canal[n+i][n+j];
               aux->canal[n+i][n-j]=aux->canal[n+i][n+j];
            }
      return aux;      
   }

CimagenBN* kernelLaplacianoG2D(double sigma)
   {
      CimagenBN *k1=kernelDerivada2GX2D(sigma);
      CimagenBN *k2=kernelDerivada2GY2D(sigma);
      CimagenBN *aux;
      aux=(*k1)+k2;
      delete k1;
      return aux;
   }



   
void CimagenBN::cuantizar(int n, double *cortes, double *valores)//supongo que es creciente!!!
   {
      max=valores[posicion(max,n,cortes)];
      min=valores[posicion(min,n,cortes)];
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]=valores[posicion(canal[i][j],n,cortes)];
   }   
   
void CimagenBN::cuantizar(int n)
   {
      double h=(max-min)/(double)n;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]=((int)((canal[i][j]-min)/h))*h+min;
   }   
   
void CimagenBN::calcularHistograma(int n, int **cont)
   {
      double **pixeles;
      double tono,h;
      max++;
      if (canal==NULL)
         return;
      h=(max-min)/(double)n;
      (*cont)=(int *)malloc(n*sizeof(int));
      for (int k=0;k<n;k++)
        (*cont)[k]=0;

      for(int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            {
               tono=canal[i][j];
               (*cont)[(int)((tono-min)/h)]++;
            }
      max--;
   }



void CimagenBN::ecualizar(int n, int *h)//ecualiza esta imagen usando el histograma h de n tonos
   {
      
      max++;
      double dx=(max-min)/(double)n;
      for (int i=1;i<n;i++)//calcula el histograma acumulado
         h[i]+=h[i-1];
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            {
               canal[i][j]=h[(int)((canal[i][j]-min)/dx)];
            }
      min=h[0];
      max=h[n-1];
   }
void CimagenBN::aplicarFuncion(double (*f)(double))
   {
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]=f(canal[i][j]);
   }

void CimagenBN::aplicarFuncion(double (*f)(double, double))
   {
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            canal[i][j]=f(j,i);
   }



CimagenBN* CimagenBN::convolucionNoCircular(CimagenBN& IC)
   {
      CimagenBN *aux=new CimagenBN(largo, alto);
      int centroFil=IC.alto/2, centroCol=IC.largo/2;//la ubicacion del impulso para dar la respuesta IC
      int modFil=IC.alto%2;
      int modCol=IC.largo%2;

      double **gk=IC.canal;//seal auxiliar, kernel

      for (int i=0;i<IC.alto;i++)
         gk[i]+=centroCol;
      gk+=centroFil;

      for (int i=0;i<alto;i++)
         {
            int k0=i-centroFil+1-modFil;//si modFil=1, entonces hay un pixel mas en la parte de arriba
               if(k0<0)
                  k0=0;
            int k1=i+centroFil;
               if(k1 > (alto-1))
                  k1=alto-1;
            for (int j=0;j<largo;j++)
               {

                  int L0=j-centroCol+1-modCol;
                     if(L0<0)
                        L0=0;
                  int L1=j+centroCol;
                     if(L1>(largo-1))
                        L1=largo-1;
                  double suma=0;
                  for (int k=k0;k<=k1;k++)
                     for(int L=L0;L<=L1;L++)
                        suma+=canal[k][L]*gk[i-k][j-L];
                  aux->canal[i][j]=suma;
               }
         }
      gk-=centroFil;
      for (int i=0;i<IC.alto;i++)
         gk[i]-=centroCol;
      return aux;
   }

CimagenBN* CimagenBN::convolucionSeparable(CimagenBN& K1, CimagenBN& K2)//convolucion no circular por renglones
   {
      CimagenBN *aux;
      aux=convolucionNoCircular(K1);
      *aux=aux->convolucionNoCircular(K2);
      return aux;
   }

CimagenBN* CimagenBN::suavizaGaussiana(double sigma)
   {
      CimagenBN *k1=kernelGaussianoF(sigma);      
      CimagenBN *k2=kernelGaussianoC(sigma);
      CimagenBN *aux=convolucionSeparable(*k1, *k2);
      delete k1;
      delete k2;
      return aux;
   }   

double CimagenBN::maximo(void)
   {
      double maxActual=canal[0][0];
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            if (maxActual<canal[i][j])
               maxActual=canal[i][j];
      return maxActual;
   }

double CimagenBN::minimo(void)
   {
      double minActual=canal[0][0];
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            if (minActual>canal[i][j])
               minActual=canal[i][j];
      return minActual;
   }

double CimagenBN::suma(void)
   {
      double s=0;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            s+=canal[i][j];
      return s;
   }
   
double CimagenBN::media(void)
   {
      return suma()/(double)(alto*largo);
   }

double CimagenBN::varianza(void)
   {
      double m=media(), suma=0;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            suma+=(canal[i][j]-m)*(canal[i][j]-m);
      return suma/(double)(alto*largo);
   }

AnsiString CimagenBN::estadisticas(void)
   {
      AnsiString aux="";
      aux.printf("Min=%0.2f. Max=%0.2f. Suma=%0.2f. Media=%0.2f.",minimo(),maximo(),suma(),media());
      return aux;
      
   }

CimagenBN* CimagenBN::derivadaX(int dir)
//derivada parcial respecto a X NO circular.
   {
      CimagenBN *aux=new CimagenBN(largo,alto);
      if(dir==-1)//derivada hacia atras
         for (int i=0;i<alto;i++)
            {
               for (int j=1;j<largo;j++)
                  {
                     double dif=canal[i][j]-canal[i][j-1];
                     aux->canal[i][j]=dif;
                  }
               aux->canal[i][0]=aux->canal[i][1];
            }
      else if (dir==1)//derivada hacia adelante
         for (int i=0;i<alto;i++)
            {
               for (int j=0;j<(largo-1);j++)
                  aux->canal[i][j]=canal[i][j+1]-canal[i][j];
               aux->canal[i][largo-1]=aux->canal[i][largo-2];
            }
      else//derivada centrada
         for (int i=0;i<alto;i++)
            {
               for (int j=1;j<(largo-1);j++)
                  aux->canal[i][j]=canal[i][j+1]-canal[i][j-1];
               aux->canal[i][0]=aux->canal[i][1];
               aux->canal[i][largo-1]=aux->canal[i][largo-2];
            }
         
      return aux;
   }
   
CimagenBN* CimagenBN::derivadaY(int dir)
   {
      CimagenBN *aux=new CimagenBN(largo,alto);
      if(dir==-1)//derivada hacia atras
         for (int j=0;j<largo;j++)
            {
               for (int i=1;i<alto;i++)
                  aux->canal[i][j]=canal[i][j]-canal[i-1][j];
               aux->canal[0][j]=aux->canal[1][j];
            }
      else if (dir==1)//derivada hacia adelante
         for (int j=0;j<largo;j++)
            {
               for (int i=0;i<(alto-1);i++)
                  aux->canal[i][j]=canal[i+1][j]-canal[i][j];
               aux->canal[alto-1][j]=aux->canal[alto-2][j];
            }
      else
         for (int j=0;j<largo;j++)
            {
               for (int i=1;i<(alto-1);i++)
                  aux->canal[i][j]=canal[i+1][j]-canal[i-1][j];
               aux->canal[alto-1][j]=aux->canal[alto-2][j];
               aux->canal[0][j]=aux->canal[1][j];
            }
      return aux;
   }
CimagenBN* CimagenBN::derivadaGaussianaX(double sigma)
   {
      CimagenBN *kernelD, *kernelG;
      CimagenBN *aux;
      
      kernelD=kernelDerivadaGX(sigma);
      kernelG=kernelGaussianoC(sigma);
      aux=convolucionSeparable(*kernelD, *kernelG);
      delete kernelD;
      delete kernelG;
      return aux;
   }
CimagenBN* CimagenBN::derivadaGaussianaY(double sigma)
   {
      CimagenBN *kernelD, *kernelG;
      CimagenBN *aux;
      
      kernelD=kernelDerivadaGY(sigma);
      kernelG=kernelGaussianoF(sigma);
      aux=convolucionSeparable(*kernelD, *kernelG);
      delete kernelD;
      delete kernelG;
      return aux;
   }

CimagenBN* CimagenBN::derivadaGaussianaXY(double sigma)
   {
      CimagenBN *k1=kernelDerivadaGX(sigma);
      CimagenBN *k2=kernelDerivadaGY(sigma);
      CimagenBN *aux;

      aux=convolucionNoCircular(*k1);
      (*aux)=aux->convolucionNoCircular(*k2);
      delete k1;
      delete k2;
      return aux;
   }

CimagenBN* CimagenBN::derivadaGaussiana2X(double sigma)
   {
      CimagenBN *kernelD, *kernelG;
      CimagenBN *aux;
      
      kernelD=kernelDerivada2GX(sigma);
      kernelG=kernelGaussianoC(sigma);
      aux=convolucionSeparable(*kernelD, *kernelG);
      delete kernelD;
      delete kernelG;
      return aux;
   }

CimagenBN* CimagenBN::derivadaGaussiana2Y(double sigma)
   {
      CimagenBN *kernelD, *kernelG;
      CimagenBN *aux;
      
      kernelD=kernelDerivada2GY(sigma);
      kernelG=kernelGaussianoF(sigma);
      aux=convolucionSeparable(*kernelD, *kernelG);
      delete kernelD;
      delete kernelG;
      return aux;
   }

CimagenBN* CimagenBN::derivadaGaussiana3X(double sigma)
   {
      CimagenBN *kernelD, *kernelG;
      CimagenBN *aux;
      
      kernelD=kernelDerivada3GX(sigma);
      kernelG=kernelGaussianoC(sigma);
      aux=convolucionSeparable(*kernelD, *kernelG);
      delete kernelD;
      delete kernelG;
      return aux;
   }

CimagenBN* CimagenBN::derivadaGaussiana3Y(double sigma)
   {
      CimagenBN *kernelD, *kernelG;
      CimagenBN *aux;
      
      kernelD=kernelDerivada3GY(sigma);
      kernelG=kernelGaussianoF(sigma);
      aux=convolucionSeparable(*kernelD, *kernelG);
      delete kernelD;
      delete kernelG;
      return aux;
   }
//**********************************para el problema particular de la tarea 10*********************
CimagenBN* CimagenBN::derivadaGaussiana2XY(double sigma)
   {
      CimagenBN *kernelX, *kernelY;
      CimagenBN *aux;
      
      kernelX=kernelDerivada2GX(sigma);
      kernelY=kernelDerivadaGY(sigma);
      aux=convolucionSeparable(*kernelX, *kernelY);
      delete kernelX;
      delete kernelY;
      return aux;
   }

CimagenBN* CimagenBN::derivadaGaussiana2YX(double sigma)
   {
      CimagenBN *kernelX, *kernelY;
      CimagenBN *aux;

      kernelX=kernelDerivadaGX(sigma);
      kernelY=kernelDerivada2GY(sigma);
      aux=convolucionSeparable(*kernelX, *kernelY);
      delete kernelX;
      delete kernelY;
      return aux;
   }
//*************************************************************************************************
CimagenBN* CimagenBN::laplacianoGaussiano(double sigma)
   {
      CimagenBN *aux;
      CimagenBN *kernel;
      kernel=kernelLaplacianoG2D(sigma);
      aux=convolucionNoCircular(*kernel);
      delete kernel;
      return aux;
   }

CimagenBN* CimagenBN::magnitudGradienteG(double sigma)
   {
      CimagenBN *aux=new CimagenBN(largo, alto);
      CimagenBN *dx;
      CimagenBN *dy;
      
      dx=derivadaGaussianaX(sigma);
      dy=derivadaGaussianaY(sigma);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=(dx->canal[i][j]*dx->canal[i][j])+(dy->canal[i][j]*dy->canal[i][j]);
      delete dx;
      delete dy;
      return aux;
   }

CimagenBN* CimagenBN::umbralPorcentual(double u)
   {
      CimagenBN *aux=new CimagenBN(largo, alto);
      max=maximo();
      min=minimo();
      double valCorte=min+(max-min)*u;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=(canal[i][j]>valCorte?1:0);
      return aux;
   }

CimagenBN* CimagenBN::umbral(double u)
   {
      CimagenBN *aux=new CimagenBN(largo, alto);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=(canal[i][j]>=u?1:0);
      return aux;
   }

CimagenBN* CimagenBN::umbralInvertido(double u)
   {
      CimagenBN *aux=new CimagenBN(largo, alto);
      max=-INFINITO;
      min=INFINITO;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            {
               if(canal[i][j]<min)
                  min=canal[i][j];
               if(canal[i][j]>max)
                  max=canal[i][j];
            }
      double valCorte=min+(max-min)*u;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=(canal[i][j]<valCorte?1:0);
      return aux;
   }

CimagenBN* CimagenBN::umbralFijoInvertido(double u)
   {
      CimagenBN *aux=new CimagenBN(largo, alto);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            aux->canal[i][j]=(canal[i][j]<u?1:0);
      return aux;
   }

CimagenBN* CimagenBN::crucesPorCero(void)
{
   CimagenBN *aux=new CimagenBN(largo, alto);
   double **dat=canal, **res=aux->canal;
   int nr=alto, nc=largo;
   
   for (int i=0;i<nr;i++)
      for (int j=0;j<nc;j++)
         {
            if(i && ( (dat[i][j]*dat[i-1][j] < 0) || (i-1 && dat[i][j]*dat[i-1][j] == 0 && dat[i-1][j]*dat[i-2][j]!=0) )  )
            res[i][j] = 1;
            else
            {
            if (j && ( (dat[i][j]*dat[i][j-1] < 0) || (j-1 && dat[i][j]*dat[i][j-1] == 0 && dat[i][j-1]*dat[i][j-2]!=0)) )
            	 res[i][j] = 1;
            else
                      res[i][j] = 0;
            }
         }
   return aux;		
}

CimagenBN* CimagenBN::detectarBordes(double sigma, double u)
   {
      CimagenBN *MG;//magnitud del gradiente
      CimagenBN *Lap;//laplaciano gaussiano(sigma)
      MG=magnitudGradienteG(sigma);
      Lap=laplacianoGaussiano(sigma);
      (*MG)=MG->umbral(u);
      (*Lap)=Lap->crucesPorCero();
      (*Lap)*=(*MG);
      delete MG;
      return Lap;
   }

CimagenBN* CimagenBN::derivadaDireccionalMG(double sigma, double u)
   {
      CimagenBN *dx, *dx2, *dy, *dy2, *dxx, *dyy, *dxy, *aux;
      dx=derivadaGaussianaX(sigma);
      dx2=(*dx) *(*dx);
      dy=derivadaGaussianaY(sigma);
      dy2=(*dy) *(*dy);
      dxx=derivadaGaussiana2X(sigma);
      dyy=derivadaGaussiana2Y(sigma);
      dxy=derivadaGaussianaXY(sigma);      
      aux=*(*((*dx2)*(*dxx))   +   (*(   (*((*dx)*(*dy))*(*dxy))   )*2)) + (*dy2)*(*dyy);
      delete dx;
      delete dx2;
      delete dxx;
      delete dy;
      delete dy2;
      delete dyy;
      delete dxy;
      return aux;
   }

CimagenBN* CimagenBN::segundaDerivadaDireccionalMG(double sigma, double u)
   {
      CimagenBN *dx, *dx2, *dx3, *dy, *dy2, *dy3, *dxxx, *dyyy, *dxxy, *dyyx, *aux;
      
      dx=derivadaGaussianaX(sigma);
      dx2=(*dx) *(*dx);
      dx3=(*dx2)*(*dx);
      dy=derivadaGaussianaY(sigma);
      dy2=(*dy) *(*dy);
      dy3=(*dy2)*(*dy);
      dxxx=derivadaGaussiana3X(sigma);
      dyyy=derivadaGaussiana3Y(sigma);
      dxxy=derivadaGaussiana2XY(sigma);
      dyyx=derivadaGaussiana2YX(sigma);
      
      aux=(*((*((*((*dx3)*(*dxxx)))+((*((*((*dx2)*(*dy)))*(*dxxy)))*3)))+((*((*((*dx)*(*dy2)))*(*dyyx)))*3)))+((*dy3)*(*dyyy));
      return aux;   
   }

CimagenBN* CimagenBN::detectorDireccional(double sigma, double u)
   {
      CimagenBN *dx, *dx2, *dx3, *dy, *dy2, *dy3, *dxx, *dyy, *dxy, *dxxx, *dyyy, *dxxy, *dyyx, *aux, *aux2;
      dx=derivadaGaussianaX(sigma);
      dx2=(*dx) *(*dx);
      dx3=(*dx2)*(*dx);
      dy=derivadaGaussianaY(sigma);
      dy2=(*dy) *(*dy);
      dy3=(*dy2)*(*dy);
      
      dxx=derivadaGaussiana2X(sigma);
      dyy=derivadaGaussiana2Y(sigma);
      dxy=derivadaGaussianaXY(sigma);
      dxxx=derivadaGaussiana3X(sigma);
      dyyy=derivadaGaussiana3Y(sigma);
      dxxy=derivadaGaussiana2XY(sigma);
      dyyx=derivadaGaussiana2YX(sigma);

      aux=*(*((*dx2)*(*dxx))   +   (*(   (*((*dx)*(*dy))*(*dxy))   )*2)) + (*dy2)*(*dyy);
      (*aux)=aux->crucesPorCero();
      //*******ahora la otra imagen**********
      aux2=(*((*((*((*dx3)*(*dxxx)))+((*((*((*dx2)*(*dy)))*(*dxxy)))*3)))+((*((*((*dx)*(*dy2)))*(*dyyx)))*3)))+((*dy3)*(*dyyy));
      (*aux2)=aux2->umbralInvertido(u);
      /*
           ((*dx3)*(*dxxx))
           ((*((*((*dx2)*(*dy)))*(*dxxy)))*3)
           ((*((*((*dx)*(*dy2)))*(*dyyx)))*3)
           ((*dy3)*(*dyyy))
      */
      delete dx;
      delete dx2;
      delete dx3;
      delete dxx;
      delete dxxx;
      delete dy;
      delete dy2;
      delete dy3;
      delete dyy;
      delete dyyy;
      delete dxy;
      delete dxxy;
      delete dyyx;
      CimagenBN *solucion;
      solucion=(*aux2)*(*aux);
      delete aux;
      delete aux2;
      return solucion;
   }

CimagenBN* CimagenBN::correlacionCruzada(CimagenBN* h)//h es el kernel
   {
      CimagenBN* aux=new CimagenBN(largo, alto);
      int cf=h->alto/2, cc=h->largo/2;
      double suma;
      for (int i=0;i<alto;i++)
         for(int j=0;j<largo;j++)
           {
              int kIni=-MIN(i,cf);
              int lIni=-MIN(j,cc);
              int kFin=MIN(alto-i,h->alto-cf);
              int lFin=MIN(largo-j, h->largo-cc);
              suma=0;
              for (int k=kIni;k<kFin;k++)
                 for (int l=lIni;l<lFin;l++)
                    suma+=canal[i+k][j+l]*h->canal[cf+k][cc+l];
              aux->canal[i][j]=suma;
           }
      return aux;
   }

CimagenBN* CimagenBN::dilatacion(CimagenBN* h)//h es el kernel
   {
      CimagenBN* aux=new CimagenBN(largo, alto);
      int cf=h->alto/2, cc=h->largo/2;
      double suma;
      int intNoVacia;
      for (int i=0;i<alto;i++)
         for(int j=0;j<largo;j++)
           {
              int kIni=-MIN(i,cf);
              int lIni=-MIN(j,cc);
              int kFin=MIN(alto-i,h->alto-cf);
              int lFin=MIN(largo-j, h->largo-cc);
              intNoVacia=0;
              for (int k=kIni;k<kFin && !intNoVacia;k++)
                 for (int l=lIni;l<lFin && !intNoVacia;l++)
                    if(canal[i+k][j+l]*h->canal[cf+k][cc+l]>0)
                       intNoVacia=1;
              aux->canal[i][j]=intNoVacia;
           }
      return aux;
   }

CimagenBN* CimagenBN::erosion(CimagenBN* h)//h es el kernel
   {
      CimagenBN* aux=new CimagenBN(largo, alto);
      int cf=h->alto/2, cc=h->largo/2;
      double suma;
      int intNoCompleta;
      for (int i=0;i<alto;i++)
         for(int j=0;j<largo;j++)
           {
              int kIni=-MIN(i,cf);
              int lIni=-MIN(j,cc);
              int kFin=MIN(alto-i,h->alto-cf);
              int lFin=MIN(largo-j, h->largo-cc);
              intNoCompleta=0;
              for (int k=kIni;k<kFin && !intNoCompleta;k++)
                 for (int l=lIni;l<lFin && !intNoCompleta;l++)
                    if(canal[i+k][j+l]==0 && h->canal[cf+k][cc+l]==1)
                       intNoCompleta=1;
              aux->canal[i][j]=!intNoCompleta;
           }
      return aux;
   }



const int maxColores=10000;
int minimoColor(int c, int *P)
   {
      if(P[c]==c)
         return c;
      else
         return minimoColor(P[c], P);
   }

void reconstruirPaleta(int *P, int n)
   {
      for (int i=0;i<n;i++)
         P[i]=minimoColor(P[i], P);
   }
CimagenBN* CimagenBN::clasificar(void)
   {
      CimagenBN *aux;
      int c=largo, f=alto;
      
      int *color=(int*)malloc(maxColores*sizeof(int));
      int menor, mayor;
      for (int i=0;i<maxColores;i++)
         color[i]=i;

      aux=new CimagenBN(largo, alto);
      //***procesar la primera fila***
      aux->canal[0][0]=1;//la clase cero esta reservada
      int colActual=1;
      for (int i=1;i<c;i++)
         {
            if(canal[0][i]!=canal[0][i-1])//cambio de color
               {
                  colActual++;
                  aux->canal[0][i]=colActual;
               }
            else
               aux->canal[0][i]=aux->canal[0][i-1];
            
         }
      //********************************

      for (int i=1;i<f;i++)
         {
            //procesar la primera celda de cada fila
            if(canal[i][0]==canal[i-1][0])//igual a la de arriba
               aux->canal[i][0]=aux->canal[i-1][0];
            else
               {
                  colActual++;
                  aux->canal[i][0]=colActual;
               }
            
            for (int j=1;j<c;j++)
               {
                  if((canal[i-1][j]==canal[i][j]) && (canal[i][j]==canal[i][j-1]))//los tres iguales
                     {
                        menor=MIN(color[(int)aux->canal[i-1][j]],color[(int)aux->canal[i][j-1]]);
                        mayor=MAX(color[(int)aux->canal[i-1][j]],color[(int)aux->canal[i][j-1]]);
                        color[mayor]=menor;
                        aux->canal[i][j]=menor;
                     }
                  else if(canal[i][j]==canal[i][j-1])//igual que el de la izquierda
                     aux->canal[i][j]=color[(int)aux->canal[i][j-1]];
                  else if(canal[i][j]==canal[i-1][j])//igual que el de arriba
                     aux->canal[i][j]=color[(int)aux->canal[i-1][j]];
                  else
                     {
                        colActual++;
                        aux->canal[i][j]=colActual;
                     }
               }
         }
      //ahora construir los colores verdaderos
      reconstruirPaleta(color,maxColores);
      for (int i=0;i<f;i++)
         for (int j=0;j<c;j++)
            if (canal[i][j]==1)//blanco
               aux->canal[i][j]=0;
            else
               aux->canal[i][j]=color[(int)aux->canal[i][j]];

      //liberar memoria
      free(color);
      return aux;
   }

int posVal(int *A, int n, int v)
   {
      
      for (int i=0;i<n;i++)
         if(A[i]==v)
            return i;
      return -1;
   }


CimagenBN* CimagenBN::dilatacionGeneral(CimagenBN* h)
   {
      CimagenBN *aux=new CimagenBN(largo, alto);
      int numColores=0;
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            if ((int)canal[i][j]>numColores)
               numColores=(int)canal[i][j];
      numColores++;
      
      int *cont=(int *)malloc(numColores*sizeof(int));
      for (int i=0;i<numColores;i++)
         cont[i]=0;

      int cf=h->alto/2, cc=h->largo/2;
      for (int i=0;i<alto;i++)
         for(int j=0;j<largo;j++)
           {
              int kIni=-MIN(i,cf);
              int lIni=-MIN(j,cc);
              int kFin=MIN(alto-i,h->alto-cf);
              int lFin=MIN(largo-j, h->largo-cc);

              for (int k=kIni;k<kFin;k++)
                 for (int l=lIni;l<lFin;l++)
                    if((canal[i+k][j+l]!=0) && (h->canal[cf+k][cc+l]!=0))
                       cont[(int)canal[i+k][j+l]]++;
              
              int maxCont=0;
              for (int i=1;i<numColores;i++)
                 if (cont[maxCont]<cont[i])         
                    maxCont=i;
              aux->canal[i][j]=maxCont;
              for (int i=0;i<numColores;i++)
                 cont[i]=0;
           }
        free(cont);
        return aux;
   }

void CimagenBN::recuperarLinea(void)
   {
      int kIni, kFin, lIni, lFin, minCol;
      bool variosColores;
      
      for (int i=0;i<alto;i++)
         for(int j=0;j<largo;j++)
            if(canal[i][j]!=0)
               {
                  kIni=i>0?-1:0;
                  kFin=i<(alto-1)?1:0;
                  lIni=j>0?-1:0;
                  lFin=j<(largo-1)?1:0;
                  variosColores=false;
                  minCol=canal[i][j];
                  int aux;
                  for(int k=kIni;k<=kFin;k++)
                     for(int l=lIni;l<=lFin;l++)
                        {
                           aux=canal[i+k][j+l];
                           if(aux!=0 && minCol!=aux)
                              {
                                 variosColores=true;
                                 if(aux<minCol)
                                    minCol=aux;
                              }
                        }
                  if(variosColores && (canal[i][j]==minCol))
                  //if(variosColores)
                     canal[i][j]=0;
               }
   }

double CimagenBN::sumaPotencialVecinos(int f, int c, double (*V)(double))
   {
      double suma=0;
      if(f>0)
         suma+=V(canal[f][c]-canal[f-1][c]);
      if(f<(alto-1))
         suma+=V(canal[f][c]-canal[f+1][c]);
      if(c>0)
         suma+=V(canal[f][c]-canal[f][c-1]);
      if(c<(largo-1))
         suma+=V(canal[f][c]-canal[f][c+1]);
      return suma;
   }

int CimagenBN::numVecinos(int f, int c)
   {
      int suma=2;
      if(f>0 && f<(alto-1))
         suma++;
      if(c>0 && c<(largo-1))
         suma++;
      return suma;
   }

CimagenBN* CimagenBN::conRuidoG(double m, double s)
   {
      CimagenBN *aux=new CimagenBN(largo, alto);
      for (int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            {
               aux->canal[i][j]+=canal[i][j]+RandG(m,s);
            }
      return aux;
   }

double CimagenBN::magnitud(void)
   {
      double suma=0;
      for(int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            suma+=canal[i][j]*canal[i][j];
      return sqrt(suma);
   }

double derivadaH(CimagenBN *Img, int x, int y)
   {
      if(x>0 && x<(Img->largo-1) && 0<=y && y<Img->alto)
         return 2*(Img->canal[y][x])-Img->canal[y][x-1]-Img->canal[y][x+1];
      else
         return 0;
   }

double derivadaV(CimagenBN *Img, int x, int y)
   {
      if(y>0 && y<(Img->alto-1) && 0<=x && x<Img->largo)
         return 2*Img->canal[y][x]-Img->canal[y-1][x]-Img->canal[y+1][x];
      else
         return 0;
   }

double derivadaC(CimagenBN *Img, int x, int y)
   {
      if(x>0 && y>0 && x<Img->largo && y<Img->alto)
         {
            return Img->canal[x-1][y-1]-Img->canal[x][y-1]+Img->canal[x][y]-Img->canal[x-1][y];
         }
      else
         return 0;
   }

CimagenBN* CimagenBN::gradienteResortes(CimagenBN *obs, double lambda, double (*V)(double))
   {
      CimagenBN *aux=new CimagenBN(largo, alto);
      double buffer;

      for(int i=0;i<alto;i++)
         for (int j=0;j<largo;j++)
            {
               buffer=2*((canal[i][j])-(obs->canal[i][j]))+lambda*(sumaPotencialVecinos(i,j,V));
               aux->canal[i][j]=buffer;
            }
      return aux;
   }

CimagenBN* CimagenBN::gradientePlacas(CimagenBN *obs, double lambda)
  {
     CimagenBN *aux=new CimagenBN(largo, alto);

     for(int y=0;y<alto;y++)
        for(int x=0;x<largo;x++)
           {
              aux->canal[y][x]=2*derivadaH(this, x,y)-derivadaH(this, x-1,y)-derivadaH(this, x+1,y)
                              +2*derivadaV(this, x,y)-derivadaV(this, x,y-1)-derivadaV(this, x,y+1)
                              +2*(derivadaC(this, x,y)      - derivadaC(this, x-1,y)
                              +  derivadaC(this, x-1, y+1) + derivadaC(this, x,y-1));
              aux->canal[y][x]*=lambda;
              aux->canal[y][x]+= canal[y][x]-obs->canal[y][x];
           }
     return aux;
  }
#endif


