#ifndef eigenH
#define eigenH
#include "algoritmillos.h"
#include <alloc.h>
#include <grids.hpp>
#include <stdio.h>
#include "vector.h"

class Ceigen
   {
      public:
         Cvector **vector;//arreglo de apuntadores a vector
         double* valor;//arreglo de double's
         int dim, n;
         Ceigen();
         ~Ceigen();
         Ceigen(int d);
         void pedirMemoria(int tam);
         void liberarMemoria(void);
         void agregar(double val, Cvector *vect);
         void imprimir(TStringGrid *s);
         void guardar(char *arch);
         void cargar(char *arch);
         double quitarInfluenciaOrtogonal(Cvector *vect);
   };

void Ceigen::pedirMemoria(int tam)
   {
      n=0;
      dim=tam;
      vector=(Cvector **)malloc(dim*sizeof(Cvector *));
      for (int i=0;i<dim;i++)
         vector[i]=NULL;
      valor=(double *)malloc(dim*sizeof(double));
   }

void Ceigen::liberarMemoria(void)
   {
      if (vector!=NULL)
         for (int i=0;i<n;i++)            
            free(vector[i]);
         free(vector);
      if (valor!=NULL)
         free(valor);
   }

Ceigen::Ceigen()
   {
      dim=n=0;
      valor=NULL;
      vector=NULL;
   }

Ceigen::~Ceigen()
   {
      liberarMemoria();
   }

Ceigen::Ceigen(int d)
   {
      pedirMemoria(d);
   }

void Ceigen::agregar(double val, Cvector *vect)
   {
      if (n<dim)//hay espacio...
         {
            (vector[n])=new Cvector(dim);
            *(vector[n])=*vect;
            valor[n]=val;
            n++;
         }
      
   }

void Ceigen::imprimir(TStringGrid *s)
   {
      s->RowCount=dim+1;
      s->ColCount=dim+1;
      s->FixedCols=1;
      s->FixedRows=1;
      s->Cells[0][0]="Valores propios";
      for (int i=1;i<=dim;i++)//titulos de columnas
         s->Cells[i][0]=(AnsiString("v[")+(i-1))+"]";                     
      for (int i=0;i<n;i++)//valores propios en la primera columna   
         s->Cells[0][i+1]=valor[i];
      for (int i=0;i<n;i++)//para cada vector propio
         for (int j=0;j<dim;j++)
            s->Cells[j+1][i+1]=vector[i]->M[j];            
   }

void Ceigen::guardar(char *arch)
   {
      if (arch!=NULL && arch[0]!=0)
         {
            FILE *F;
            F=fopen(arch,"w");
            fprintf(F,"%d%d\n",dim,n);
            for (int i=0;i<n;i++)
               {
                  for (int j=0;j<dim;j++)
                     fprintf(F,"%f ",vector[i]->M[j]);
                  fprintf(F,"\n");
               }
            fclose(F);
         }
   }

void Ceigen::cargar(char *arch)
   {
      if (arch!=NULL && arch[0]!=0)
         {
            FILE *F;
            float aux;
            F=fopen(arch,"r");
            fscanf(F,"%d%d",&dim,&n);
            for (int i=0;i<n;i++)
               for (int j=0;j<dim;j++)
                 {
                    fscanf(F,"%f",&aux);
                    vector[i]->M[j]=aux;
                 }
            fclose(F);
         }
   }
double Ceigen::quitarInfluenciaOrtogonal(Cvector *vect)
//si los vectores propios son ortonormales, entonces v=suma(a_i*v_i),i=n..dim-1, los demas son ceros...i.e.
//hace que al final v sea su proyeccion en el complemento de la suma directa de los espacios propios generados
//por los vectores propios de *this. Normaliza v_final y regresa su norma
   {
      double ak;
      Cvector influencia(dim);
      influencia.cero();
      for (int k=0;k<n;k++)
         {
            ak=*(vector[k])*(*vect);//el producto punto
            influencia+=((*vector[k])*ak);
         }
      *vect-=influencia;
      return vect->normalizar(EUCLIDEANA);
      //y sha'sta!
   }
   
   
   
   
   
   
   
   
   
   
   
   
#endif //eigenH


