#ifndef ExtCtrlsHPP
#include <ExtCtrlsHPP>
#endif	//ExtCtrlsHPP
#include "algoritmillos.h"
#ifndef graficacionH
#define graficacionH
#include <alloc.h>
#include <math.h>
struct SpuntoInt
   {
      int px,py;
   };
struct SpuntoFloat
   {
      double px, py;
   };
class Cventana
   {


       public:
          #define numSeparaciones 5;
          int largo, alto;
          TImage *Imagen;
          double minX, maxX, minY, maxY;
          double saltoX, saltoY;
          double xrate, yrate;//"tamanio" de cada pixel
          double minTono, maxTono, proporcion;
          bool separaciones;
          TColor col;

          Cventana(TImage *Im, double mX=-100, double MX=100,  double mY=-100, double MY=100);
          void coordReales(int x, int y, double *Rx, double *Ry);
          void coordImagen(double x, double y, int *Ix, int *Iy);
          void dibujarPunto(double px, double py);
          void dibujarMancha(double px, double py);

          void dibujarLinea(double iniX, double iniY,double finX, double finY);
          void graficarFuncion( double(*f)(double), double iniX, double finX);
          void graficarNiveles(double(*f)(double, double), double iniX, double iniY, double finX, double finY);

          void dibujarEjes();
          void dibujarTexto(AnsiString cad, double px, double py);
          void limpiar(void);
          void mover(double dx, double dy);
          void fijarCentro(double px, double py);
          void fijarVentana(double mX=-100,  double mY=-100, double MX=100, double MY=100);
          void dibujarTriangulo(double **P);
          void dibujarPoligono(int n, double **P);
          void dibujarHistograma(int n, int *cont);
          void dibujarBarra(float x1,float y1,float x2,float y2);
          void curvasDeNivel(double (*f)(double *, int), int nCurvas, double iniX, double iniY, double finX, double finY);
          //debe determinar la longitud optima de salto
   };

void Cventana::coordReales(int x, int y, double *Rx, double *Ry)
   {
      *Rx=minX+(x*xrate);
      *Ry=maxY-(y*yrate);
   }
void Cventana::coordImagen(double x, double y, int *Ix, int *Iy)
   {
      (*Ix)=(int)((x-minX)/xrate);
      (*Iy)=(int)((maxY-y)/yrate);
   }

Cventana::Cventana(TImage *Im, double mX, double MX,  double mY, double MY)
   {
      Imagen=Im;
      largo=Imagen->Width;
      alto =Imagen->Height;
      col=clBlack;
      separaciones=true;
      fijarVentana(mX,mY,MX,MY);
      TFont *F;
      F=Imagen->Canvas->Font;
      F->Size=10;
      F->Color=clRed;
      F->Style<<fsBold;
      F->Name="Times New Roman";
   }
void Cventana::limpiar(void)
   {
      Imagen->Canvas->FillRect(Imagen->Canvas->ClipRect);
   }
void Cventana::mover(double dx, double dy)
   {
      minX+=dx;
      maxX+=dx;
      minY+=dy;
      maxY+=dy;
   }
void Cventana::fijarCentro(double px, double py)
   {
      double distX=(maxX-minX)/2.0, distY=(maxY-minY)/2.0;

      minX=px-distX;
      maxX=px+distX;
      minY=py-distY;
      maxY=py+distY;
   }
void Cventana::fijarVentana(double mX,  double mY, double MX, double MY)
   {
      minX=mX;
      maxX=MX;
      minY=mY;
      maxY=MY;
      saltoX=(maxX-minX)/numSeparaciones;
      saltoY=(maxY-minY)/numSeparaciones;
      xrate=(maxX-minX)/largo;
      yrate=(maxY-minY)/alto;

   }
void Cventana::dibujarPunto(double px, double py)
   {
      int x, y;
      coordImagen(px,py,&x,&y);
      Imagen->Canvas->Pixels[x][y]=col;
   }
void Cventana::dibujarMancha(double px, double py)
   {
      int x, y;
      coordImagen(px,py,&x,&y);
      for (int i=x-1;i<=x+1;i++)
         for (int j=y-1;j<=y+1;j++)
            Imagen->Canvas->Pixels[i][j]=col;
            
   }

void Cventana::dibujarTexto(AnsiString cad, double px, double py)
   {
      
      int pixX, pixY;
      coordImagen(px,py,&pixX,&pixY);
      Imagen->Canvas->TextOut(pixX,pixY,cad);
   }

void Cventana::dibujarEjes()
   {
      TColor temp=col;//para reestablecer despues el color
      if (separaciones)
         {


            double px,py;

            col=clLtGray;
            px=((int)(minX/saltoX))*saltoX;
            while (px<=maxX)
               {
                  dibujarTexto(AnsiString(px), px,0);
                  dibujarLinea(px,minY,px,maxY);
                  px+=saltoX;

               }
            py=((int)(maxY/saltoY))*saltoY;
            while (py>=minY)
               {
                  dibujarTexto(AnsiString(py),0,py);
                  dibujarLinea(minX,py,maxX,py);
                  py-=saltoY;
               }


         }
      col=clBlack;
      dibujarLinea(minX, 0, maxX, 0);
      dibujarLinea(0, minY, 0, maxY);
      col=temp;
   }

void Cventana::dibujarLinea(double iniX, double iniY,double finX, double finY)
   {
      int inix,iniy,finx,finy;
      coordImagen(iniX,iniY,&inix,&iniy);
      coordImagen(finX,finY,&finx,&finy);
      Imagen->Canvas->Pen->Color=col;
      Imagen->Canvas->PenPos=TPoint(inix,iniy);
      Imagen->Canvas->LineTo(finx,finy);

   }
void Cventana::graficarFuncion( double(*f)(double), double iniX, double finX)
   {
      double salto=(maxX-minX)/(double)largo;
      double px=iniX;
      double py=f(px);
      int pixX, pixY;
      
      coordImagen(px, py, &pixX,&pixY);
      minTono=maxTono=py;
      Imagen->Canvas->Pen->Color=col;
      Imagen->Canvas->PenPos=TPoint(pixX,pixY);
      while (px<finX)
         {
            px+=salto;
            py=f(px);
            if (py<minTono)
               minTono=py;
            if (py>maxTono)
               maxTono=py;
            coordImagen(px, py, &pixX,&pixY);
            Imagen->Canvas->LineTo(pixX,pixY);
         }
   }

void Cventana::graficarNiveles(double(*f)(double, double), double iniX, double iniY, double finX, double finY)
   {
      double **val;
      double px,py;
      double saltox, saltoy;
      
      int pixx1, pixx2, pixy1, pixy2;

      coordImagen(iniX, iniY, &pixx1, &pixy2);
      coordImagen(finX, finY, &pixx2, &pixy1);
      val=(double **)malloc(alto*sizeof(double*));
      for (int i=0;i<alto;i++)
         val[i]=(double*)malloc(largo*sizeof(double));
      minTono=90000000;
      maxTono=-90000000;
      
      saltox=(finX-iniX)/(double)(pixx2-pixx1);
      saltoy=(finY-iniY)/(double)(pixy2-pixy1);
      
      px=iniX;
      py=finY;
      for (int i=pixy1;i<pixy2;i++)//para cada fila
         {
            for (int j=pixx1;j<pixx2;j++)//para cada columna
               {
                  //coordReales(j,i,&px,&py);
                  val[i][j]=f(px,py);
                  if (val[i][j]<minTono)
                     minTono=val[i][j];
                  else if (val[i][j]>maxTono)
                     maxTono=val[i][j];
                  px+=saltox;
               }
            px=iniX;
            py-=saltoy;
         }
      if (maxTono>minTono)
         proporcion=255.0/(maxTono-minTono);
      else
         proporcion=0;

      for (int i=pixy1;i<pixy2;i++)//para cada fila
         for (int j=pixx1;j<pixx2;j++)//para cada columna
            {
               Imagen->Canvas->Pixels[j][i]=gris((int)(proporcion*(val[i][j]-minTono)));
            }

      for (int k=0;k<alto; k++)
         free(val[k]);
      free(val);
   }

void Cventana::dibujarTriangulo(double **P)
   {
      for (int i=0;i<3;i++)
         dibujarLinea(P[i][0],P[i][1],P[(i+1)%3][0],P[(i+1)%3][1]);
   }
   
void Cventana::dibujarPoligono(int n,double **P)
   {
      for (int i=0;i<n;i++)
         dibujarLinea(P[i][0],P[i][1],P[(i+1)%n][0],P[(i+1)%n][1]);
   }
   
void Cventana::dibujarBarra(float x1,float y1,float x2,float y2)
   {
      dibujarLinea(x1,y1,x2,y1);
      dibujarLinea(x2,y1,x2,y2);
      dibujarLinea(x2,y2,x1,y2);
      dibujarLinea(x1,y2,x1,y1);
   }
   
void Cventana::dibujarHistograma(int n, int *cont)   
   {
      if ((n<=0) || (cont==NULL))
         return;
      
      int maximo=cont[0];
      for (int i=1;i<n;i++)
         if (cont[i]>maximo)
            maximo=cont[i];
      fijarVentana(0,0, n, maximo);
      float salto,h;
      float P[4][2];//los vertices de la barra
      
      salto=maxY/(float)maximo;
      h=maxX/(float)n;
      
      dibujarEjes();
      for (int i=0;i<n;i++)         
         dibujarBarra(i*h,cont[i]*salto,(i+1)*h,0);
   }
      
      
void Cventana::curvasDeNivel(double (*f)(double *, int), int nCurvas, double iniX, double iniY, double finX, double finY)
   {
      double **val;
      double px,py;
      double saltox, saltoy;
      double *args=(double *)malloc(2*sizeof(double));
      
      int pixx1, pixx2, pixy1, pixy2;

      coordImagen(iniX, iniY, &pixx1, &pixy2);
      coordImagen(finX, finY, &pixx2, &pixy1);
      val=(double **)malloc(alto*sizeof(double*));
      for (int i=0;i<alto;i++)
         val[i]=(double*)malloc(largo*sizeof(double));
      minTono=90000000;
      maxTono=-90000000;
      
      saltox=(finX-iniX)/(double)(pixx2-pixx1);
      saltoy=(finY-iniY)/(double)(pixy2-pixy1);
      
      px=iniX;
      py=finY;
      for (int i=pixy1;i<pixy2;i++)//para cada fila
         {
            for (int j=pixx1;j<pixx2;j++)//para cada columna
               {
                  //coordReales(j,i,&px,&py);
                  args[0]=px;
                  args[1]=py;
                  val[i][j]=f(args,2);
                  if (val[i][j]<minTono)
                     minTono=val[i][j];
                  else if (val[i][j]>maxTono)
                     maxTono=val[i][j];
                  px+=saltox;
               }
            px=iniX;
            py-=saltoy;
         }

         
      double epsilon=0.05;
      double h=(maxTono-minTono)/(double)nCurvas;
      double proporcion;
      
      for (int i=pixy1;i<pixy2;i++)//para cada fila
         for (int j=pixx1;j<pixx2;j++)//para cada columna
            {
               proporcion=(val[i][j]/h);
               proporcion-=((int)proporcion);
               if (fabs(proporcion)<epsilon)
                  Imagen->Canvas->Pixels[j][i]=azul(((val[i][j]-minTono)/(maxTono-minTono))*255);
            }

      for (int k=0;k<alto; k++)
         free(val[k]);
      free(val);
      free(args);
      
   }      
#endif //graficacionH


