#ifndef matricesH
#define matricesH
#include <alloc.h>
#include <math.h>
#include <grids.hpp>
#include <stdio.h>
#define MAXIMA 0
#define MAXIMO 0
#define ABSOLUTA 1
#define ABSOLUTO 1
#define EUCLIDEANA 2
#define EUCLIDIANA 2

#ifndef vectorH
class Cvector;
#endif
#ifndef eigenH
class Ceigen;
#endif

const defaultDim=3;
class CMatriz
   {
      public:
         int fil, col;
         int warning;
         double **M;
         CMatriz();
         CMatriz(int f, int c);
         CMatriz(int f, int c, double **Mat);
         ~CMatriz();
         void operator =(CMatriz &A);
         CMatriz& operator +(CMatriz &A);
         CMatriz& operator +(double r);
         CMatriz& operator -(double r);
         CMatriz& operator -(CMatriz &A);
         CMatriz& operator *(double r);
         CMatriz& operator *(CMatriz &A);
         Cvector& operator *(Cvector &v);
         double   operator %(CMatriz &A);//el producto punto
         double   operator %(double *v);//producto punto para arreglos

         void factorizarLU(CMatriz &L, CMatriz &U);
         CMatriz& inversa(void);
         CMatriz& transpuesta(void);
         Cvector& resolverSistema(Cvector& b);
         Cvector& resolverSistemaLU(Cvector &b, CMatriz *L=NULL, CMatriz *U=NULL);//pueden dar o no la factoracion
         void cero(void);
         void identidad();
         double norma2(void);
         double norma(int tipo);
         double normaInf(void);
         double normaMax(void);
         double normaAbs(void);
         double normalizar(int tipo=2);
         void imprimir(TStringGrid *SG);
         void cargar(TStringGrid *SG);
         void guardar(char *arch);
         void cargar(char *arch);
         void liberarMemoria(void);
         void pedirMemoria(int f, int c);
         void calcularSiguienteEigenMayor(Ceigen &e, Cvector *x, double tolerancia, int maxIter);
         void calcularSiguienteEigenMenor(Ceigen &e, Cvector *x, double tolerancia, int maxIter, CMatriz *L=NULL, CMatriz *U=NULL);
         CMatriz& rotarTransformacion(int i, int j);
         CMatriz& iterarJacobi(double tolerancia, int maxIter);
         double maximoAbsoluto(int *a, int *b);

   };
#ifndef vectorH
#include "vector.h"
#endif
#ifndef eigenH
#include "eigen.h"
#endif
double CMatriz::normaInf(void)
   {
      double maxActual=fabs(M[0][0]);
      for (int i=0;i<fil;i++)
         for (int j=0;j<col;j++)
            if (fabs(M[i][j])>maxActual)
               maxActual=fabs(M[i][j]);
      return maxActual;
   }

double CMatriz::normaMax(void)
   {
      double maxActual=fabs(M[0][0]);
      for (int i=0;i<fil;i++)
         for (int j=0;j<col;j++)
            if (fabs(M[i][j])>maxActual)
               maxActual=fabs(M[i][j]);
      return maxActual;
   }
double CMatriz::norma(int tipo)
   {
      switch (tipo)
         {
            case MAXIMA:
               return normaInf();

            case ABSOLUTA:
               return normaAbs();

            case EUCLIDEANA:
               return norma2();

         }
      return -1;
   }

void CMatriz::guardar(char *arch)
   {
      if (arch!=NULL && arch[0]!=0)
         {
            FILE *F;
            F=fopen(arch,"w");
            fprintf(F,"%d %d\n",fil,col);
            for (int i=0;i<fil;i++)
               {
                  for (int j=0;j<col;j++)
                     fprintf(F,"%f ",M[i][j]);
                  fprintf(F,"\n");
               }
            fclose(F);
         }
   }

void CMatriz::cargar(char *arch)
   {
      if (arch!=NULL && arch[0]!=0)
         {
            FILE *F;
            int f,c;

            F=fopen(arch,"r");
            fscanf(F,"%d%d",&f,&c);
            if (col==c && fil==f)
               {
                  float aux;
                  for (int i=0;i<fil;i++)
                     for (int j=0;j<col;j++)
                        {
                           fscanf(F,"%f",&aux);
                           M[i][j]=aux;
                        }
                  fclose(F);
                  return;
               }
            warning=1;
            fclose(F);
         }
   }

double CMatriz::normaAbs(void)
   {
      double suma=0;
      for (int i=0;i<fil;i++)
         for (int j=0;j<col;j++)
            suma+=fabs(M[i][j]);
      return suma;
   }



double CMatriz::normalizar(int tipo)
   {
      double norma;
      switch (tipo)
         {
            case MAXIMA:
               norma=normaInf();
               break;
            case ABSOLUTA:
               norma=normaAbs();
               break;
            case EUCLIDEANA:
               norma=norma2();
               break;
         }
      if (norma!=0)
         for (int i=0;i<fil;i++)
            for (int j=0;j<col;j++)
               M[i][j]/=norma;
      return norma;
   }


void CMatriz::pedirMemoria(int f, int c)
   {
      fil=f;
      col=c;
      M=(double **)malloc(fil*sizeof(double *));
      if (M==NULL)
         warning=1;

      for (int i=0;i<fil;i++)
         {
            M[i]=(double *)malloc(col*sizeof(double));
            if (M[i]==NULL)
               {
                  warning=1;
                  return;
               }
               
         }
      warning=0;
   }
CMatriz::CMatriz()
   {
      fil=col=defaultDim;
      pedirMemoria(fil,col);
   }
CMatriz::CMatriz(int f, int c, double **Mat)
   {       
       pedirMemoria(f,c);
       for (int i=0;i<f;i++)
          for (int j=0;j<c;j++)
             M[i][j]=Mat[i][j];
   }

CMatriz::CMatriz(int f, int c)
   {
      pedirMemoria(f,c);
   }

void CMatriz::liberarMemoria(void)
   {
      if (M!=NULL)
         {
            for (int i=0;i<fil;i++)
               if (M[i]!=NULL)
                  free(M[i]);
            free(M);
         }
   }
CMatriz::~CMatriz()
   {
      liberarMemoria();
   }
void CMatriz::operator =(CMatriz &A)
   {
      if (A.fil!=fil || A.col!=col)
         {
            CMatriz::~CMatriz();//liberar memoria
            CMatriz(A.fil, A.col);//re-asignar memoria
            warning=1;
         }
      for (int i=0;i<fil;i++)
         for(int j=0;j<col;j++)
            M[i][j]=A.M[i][j];

   }

CMatriz& CMatriz::operator +(CMatriz& A)
   {
      CMatriz* aux;
      if (A.fil!=fil || A.col!=col)
         warning=1;
      else
         {
            aux=new CMatriz(fil, col);
            for (int i=0;i<fil;i++)
               for(int j=0;j<col;j++)
                  aux->M[i][j]=M[i][j]+A.M[i][j];
         }
      return *aux;
   }
CMatriz& CMatriz::operator -(CMatriz &A)
   {
      CMatriz* aux;
      if (A.fil!=fil || A.col!=col)
         warning=1;
      else
         {
            aux=new CMatriz(fil, col);
            for (int i=0;i<fil;i++)
               for(int j=0;j<col;j++)
                  aux->M[i][j]=M[i][j]-A.M[i][j];
         }
      return *aux;
   }
CMatriz& CMatriz::operator *(double r)
   {
      CMatriz* aux;
      aux=new CMatriz(fil, col);
      for (int i=0; i<fil; i++)
         for (int j=0; j<col; j++)
            aux->M[i][j]=M[i][j]*r;
      return *aux;
   }
   
CMatriz& CMatriz::operator +(double r)
   {
      CMatriz* aux;
      aux=new CMatriz(fil, col);
      for (int i=0; i<fil; i++)
         for (int j=0; j<col; j++)
            aux->M[i][j]=M[i][j]+r;
      return *aux;
   }
   
CMatriz& CMatriz::operator -(double r)
   {
      CMatriz* aux;
      aux=new CMatriz(fil, col);
      for (int i=0; i<fil; i++)
         for (int j=0; j<col; j++)
            aux->M[i][j]=M[i][j]-r;
      return *aux;
   }

CMatriz& CMatriz::operator *(CMatriz& A)
   {
      CMatriz *aux;
      aux=new CMatriz(fil,A.col);
      if (col==A.fil)
         {
            double suma;
            for (int i=0;i<fil; i++)
               for (int j=0; j<A.col; j++)
                  {
                     suma=0;
                     for (int k=0;k<col; k++)
                        suma+=M[i][k]*A.M[k][j];
                     aux->M[i][j]=suma;
                  }
         }
      else
         warning=1;
      return *aux;
   }
   
Cvector& CMatriz::operator *(Cvector &v)
   {      
      Cvector *x;
      double suma;
      x=new Cvector(fil);
      if (col!=v.dim)
         {
            x->cero();
            x->warning=1;
            return *x;
         }
      for (int i=0;i<fil;i++)
         {
            suma=0;
            for (int j=0;j<col;j++)
               suma+=M[i][j]*v.M[j];
            x->M[i]=suma;
         }
      return *x;
   }   

double CMatriz::operator %(CMatriz &A)//el producto punto
   {
      if (col==1 && A.col==1 && fil==A.fil)
         {
            double suma=0;
            for (int i=0;i<fil;i++)
               suma+=M[i][0]*A.M[i][0];
            return suma;
         }
      else
         return 0;
   }

double CMatriz::operator %(double *v)//producto punto para arreglos
   {
      //la validez de *v y *this depende del usuario!!...
      if (col!=1)//no es un vector
         {
            warning=1;
            return 0;
         }
      double suma=0;
      for (int i=0;i<fil;i++)
         suma+=M[i][0]*v[i];
      return suma;
   }

void CMatriz::imprimir(TStringGrid *SG)
   {
      if (fil==SG->RowCount && col==SG->ColCount)
         {
            for (int i=0;i<fil;i++)
               for (int j=0;j<col;j++)
                  SG->Cells[j][i]=AnsiString(M[i][j]);
         }
   }

double CMatriz::norma2(void)
   {
      double suma=0;
      for (int i=0;i<fil; i++)
         for (int j=0;j<col; j++)
            suma+=M[i][j]*M[i][j];
      return sqrt(suma);
   }
void CMatriz::cero(void)
   {
      for (int i=0;i<fil; i++)
         for (int j=0;j<col; j++)
            M[i][j]=0;
   }
void CMatriz::identidad()
   {
      if (fil!=col)
         {
            warning=1;
            return;
         }
      cero();
      for (int i=0;i<fil; i++)
         M[i][i]=1;
         
   }
void CMatriz::cargar(TStringGrid *SG)
   {
      if (fil==SG->RowCount && col==SG->ColCount)
         {
            for (int i=0;i<fil;i++)
               for (int j=0;j<col;j++)
                  if (SG->Cells[j][i].IsEmpty())
                     M[i][j]=0;
                  else
                     M[i][j]=SG->Cells[j][i].ToDouble();
         }
   }
void CMatriz::factorizarLU(CMatriz &L, CMatriz &U)
   {

      if ((col==L.col)&&(fil==L.fil)&&(col==U.col)&&(fil==U.fil))
         {

            for (int i=0; i<col; i++)
               U.M[0][i]=M[0][i];//la primera fila es igual...

            if (U.M[0][0]==0)//factoracion imposible
               {
                  warning=1;
                  return;
               }
            for (int i=0; i<col; i++)
               L.M[i][0]=M[i][0]/U.M[0][0];//la primera columna de L...

            for (int i=1; i<fil; i++)
            //calcular i-esima fila de U y la i-esima columna de L
               {
                  double suma=0;
                  for (int k=0; k<i; k++)
                     suma+=L.M[i][k]*U.M[k][i];
                  U.M[i][i]=M[i][i]-suma;
                  for (int j=0; j<col; j++)//i-esima fila de U
                     if (j<i)
                        U.M[i][j]=0;
                     else if (j>i)
                        {
                           suma=0;
                           for (int k=0; k<i; k++)
                              suma+=L.M[i][k]*U.M[k][j];
                           U.M[i][j]=M[i][j]-suma;
                        }
                  L.M[i][i]=1;
                  if (U.M[i][i]==0)
                     {
                        warning=1;
                        return;
                     }
                  for (int j=0; j<fil; j++)//i-esima columna de L
                     if (j<i)
                        L.M[j][i]=0;
                     else if (j>i)
                        {
                           suma=0;
                           for (int k=0; k<i; k++)
                              suma+=L.M[j][k]*U.M[k][i];
                           L.M[j][i]=(M[j][i]-suma)/U.M[i][i];
                        }

               }

         }



   }
Cvector& CMatriz::resolverSistema(Cvector& b)
   {
      Cvector* x;
      x=new Cvector(fil);

      if (fil==b.dim && col==b.dim)
         {
            CMatriz U(fil, col);
            U=*this;
            double aux;
            //eliminacion gaussiana
            for (int pivote=0; pivote<fil; pivote++)
               {
                //buscar pivote
                for (int r=pivote;r<fil && U.M[pivote][pivote]==0;r++)
                   {
                      if (U.M[r][r]!=0 && r!=pivote)
                         {
                            double* temp;
                            double tmp;
                            temp=U.M[pivote];
                            U.M[pivote]=U.M[r];
                            U.M[r]=temp;

                            tmp=b.M[pivote];
                            b.M[pivote]=b.M[r];
                            b.M[r]=tmp;
                         }

                   }

                  if (U.M[pivote][pivote]==0)//no triangulable
                     {
                        warning=1;
                        return *x;
                     }

                  for (int i=pivote+1; i<fil; i++)//para cada fila
                     {
                        aux=U.M[i][pivote]/U.M[pivote][pivote];
                        for (int j=pivote; j<col; j++)
                           U.M[i][j]=U.M[i][j]-aux*U.M[pivote][j];
                        b.M[i]=b.M[i]-aux*b.M[pivote];
                     }
               }
            //sustitucion hacia atras...
            for (int k=fil-1; k>=0;k--)
               {
                  double suma=0;
                  for (int i=k+1;i<fil;i++)
                     suma+=U.M[k][i]*x->M[i];
                  x->M[k]=(b.M[k]-suma)/U.M[k][k];
               }
         }
      else
         warning=1;
      return *x;

   }
Cvector& CMatriz::resolverSistemaLU(Cvector &b, CMatriz *L, CMatriz *U)
   {
      bool recalcular=false;
      if (L==NULL)
         {
            L=new CMatriz(fil, col);
            recalcular=true;
         }
      if (U==NULL)
         {
            U=new CMatriz(fil, col);
            recalcular=true;
         }
      if (recalcular)
         factorizarLU(*L,*U);
      Cvector y(fil); 
      Cvector *x;//esta DEBE ser dinamica!!
      x=new Cvector(fil);
      double suma;
      //resolver "manualmente" el sistema Ly=b para y
      for (int k=0;k<fil;k++)
         {
            suma=0;
            for (int i=0;i<k;i++)
               suma+=L->M[k][i]*y.M[i];
            y.M[k]=(b.M[k]-suma)/L->M[k][k];
         }
      //ahora el sistema Ux=y para x
      for (int k=fil-1;k>=0;k--)
         {
            suma=0;
            for (int i=k+1;i<fil;i++)
               suma+=U->M[k][i]*x->M[i];
            x->M[k]=(y.M[k]-suma)/U->M[k][k];
         }
      return *x;
   }
CMatriz& CMatriz::inversa(void)
   {
      CMatriz *aux;
      aux=new CMatriz(fil,col);
      warning=0;
      if (fil==col)
         {
            Cvector x(fil), b(fil);            
            for (int i=0;i<fil;i++)
               {
                  for (int j=0;j<fil;j++)
                     if (i==j)
                        b.M[j]=1;
                     else
                        b.M[j]=0;
                  x=resolverSistema(b);
                  if (warning)
                     return *aux;
                  for (int k=0; k<fil;k++)
                     aux->M[k][i]=x.M[k];
               }
         }
      else
         warning=1;
      return *aux;
   }
CMatriz& CMatriz::transpuesta(void)
   {
      CMatriz *aux;
      aux=new CMatriz(col,fil);
      warning=0;
      for (int i=0;i<fil;i++)
         for (int j=0;j<col;j++)
            aux->M[j][i]=M[i][j];
      return *aux;
   }


void CMatriz::calcularSiguienteEigenMayor(Ceigen &e, Cvector *x, double tolerancia, int maxIter)//x es el vector inicial
   {
      if (fil!=col || e.dim!=fil)//fil==col
         {
               warning=1;
               return;
         }
      int cont=0;
      double valp;
      double error=tolerancia+1;
      Cvector y(e.dim);

      e.quitarInfluenciaOrtogonal(x);
      while (error>tolerancia && cont<maxIter)
         {
            y=(*this)*(*x);
            valp=(*x*y);
            e.quitarInfluenciaOrtogonal(&y);
            error=(y-*x).norma(EUCLIDEANA);
            cont++;
            *x=y;
         }
      e.agregar(valp,x);
   }

void CMatriz::calcularSiguienteEigenMenor(Ceigen &e, Cvector *x, double tolerancia, int maxIter, CMatriz *L, CMatriz *U)
   {
      if (fil!=col || e.dim!=fil)//fil==col
         {
               warning=1;
               return;
         }
      int cont=0;
      double valp;
      double error=tolerancia+1;
      Cvector y(e.dim);
      e.quitarInfluenciaOrtogonal(x);
      while (error>tolerancia && cont<maxIter)
         {
            y=resolverSistemaLU(*x,L,U);
            valp=*x*y;
            e.quitarInfluenciaOrtogonal(&y);
            error=(y-*x).norma(EUCLIDEANA);
            cont++;
            *x=y;
         }
      if (valp!=0)
         valp=1.0/valp;
      e.agregar(valp,x);

   }

CMatriz& CMatriz::rotarTransformacion(int i, int j)
   {
      CMatriz P(fil,col), *PT, A(fil,col);
      PT=new CMatriz(fil, col);
      A=*this;
      P.identidad();
      
      if (M[i][i]==M[j][j])
         {
            P.M[i][i]=P.M[j][j]=P.M[j][i]=sqrt(2)/2;
            P.M[i][j]=-P.M[j][i];
         }
      else
         {
            double b,c,r;
            int signo=(M[i][i]-M[j][j])<0?-1:1;
      
            b=fabs(M[i][i]-M[j][j]);
            c=2*M[i][j]*signo;
            r=sqrt(b*b+c*c);
            P.M[i][i]=P.M[j][j]=sqrt((b/r+1)/2.0);
            P.M[j][i]=c/(2*P.M[i][i]*r);
            P.M[i][j]=-P.M[j][i];
         }
      *PT=P.transpuesta();
      *this=(P*A)*(*PT);
      return *PT;
   }
double CMatriz::maximoAbsoluto(int *a, int *b)
   {

   	*a=0;
   	*b=1;
   	for (int i=0;i<fil;i++)
   	   for (int j=0;j<col;j++)
   	      if (i!=j && fabs(M[i][j])>fabs(M[*a][*b]))
   	         {
   	         	*a=i;
   	         	*b=j;
   	         }
   	return M[*a][*b];
   }

CMatriz& CMatriz::iterarJacobi(double tolerancia, int maxIter)
   {
   	double max;
   	int i,j, cont=0;
   	CMatriz *T;
   	
   	T=new CMatriz(fil, col);
        T->identidad();
   	max=maximoAbsoluto(&i,&j);
   	while (fabs(max)>tolerancia && (maxIter<0 || cont++<maxIter))
   	   {
              *T=*T*rotarTransformacion(i,j);
              max=maximoAbsoluto(&i,&j);
   	   }
   	return *T;
   }

#endif





