#ifndef vectorH
#define vectorH
#include <alloc.h>
#include "algoritmillos.h"
#include <stdio.h>

#ifndef matricesH
class CMatriz;
#endif
class Cvector
   {
      public:
         int dim;
         int warning;
         double *M;
         Cvector();
         ~Cvector();
         Cvector(int n);
         void liberarMemoria(void);
         void pedirMemoria(int n);
         void cero(void);
         double norma(int tipo);
         double normalizar(int tipo);
         double operator *(Cvector &v);
         Cvector& operator *(CMatriz &T);
         Cvector& operator *(double r);
         double operator [](int k);
         Cvector& operator =(Cvector &v);
         Cvector& operator +(Cvector &v);
         Cvector& operator -(Cvector &v);

         Cvector& operator +=(Cvector &v);
         Cvector& operator -=(Cvector &v);
         void cargar(TStringGrid *SG);
         void cargar(char *arch);
         void imprimir(TStringGrid *SG);
         void guardar(char *arch);

   };
#ifndef matricesH
#include "matrices.h"
#endif
void Cvector::guardar(char *arch)
   {
      if (arch!=NULL && arch[0]!=0)
         {
            FILE *F;
            F=fopen(arch,"w");
            fprintf(F,"%d\n",dim);
            for (int i=0;i<dim;i++)
               fprintf(F,"%f\n",M[i]);
            fclose(F);
         }
   }


void Cvector::imprimir(TStringGrid *SG)
   {
      if (dim==SG->RowCount)
         {
            for (int i=0;i<dim;i++)
               SG->Cells[0][i]=AnsiString(M[i]);
         }
   }

void Cvector::cargar(char *arch)
   {
      if (arch!=NULL && arch[0]!=0)
         {
            FILE *F;
            int f;
            
            F=fopen(arch,"r");
            fscanf(F,"%d",&f);
            if (dim==f)
               {
                  float aux;
                  for (int i=0;i<dim;i++)
                        {
                           fscanf(F,"%f ",&aux);
                           M[i]=aux;
                        }
                  fclose(F);
                  return;
               }
            warning=1;
            fclose(F);
         }
   }
   
void Cvector::cargar(TStringGrid *SG)
   {
      if (dim==SG->RowCount)
         {
            for (int i=0;i<dim;i++)
               if (SG->Cells[0][i].IsEmpty())
                  M[i]=0;
               else
                  M[i]=SG->Cells[0][i].ToDouble();
         }
   }

Cvector::Cvector()
   {
      M=NULL;
      dim=0;
      warning=0;
   }

Cvector::~Cvector()
   {
      liberarMemoria();
   }

Cvector::Cvector(int n)
   {
      warning=0;
      pedirMemoria(n);
   }

void Cvector::liberarMemoria(void)
   {
      warning=0;
      if (M!=NULL)
         free(M);
   }

void Cvector::pedirMemoria(int n)
   {
      dim=n;
      M=(double *)malloc(dim*sizeof(double));
      if (M==NULL)
         warning=1;
   }

double Cvector::norma(int tipo)
   {
      switch(tipo)
         {
            case MAXIMA:
               {
                  double maxActual=abs(M[0]);
                    for (int j=0;j<dim;j++)
                       if (abs(M[j])>maxActual)
                          maxActual=abs(M[j]);
                  return maxActual;
               }
            case ABSOLUTA:
               {
                  double suma=0;
                     for (int j=0;j<dim;j++)
                        suma+=abs(M[j]);
                  return suma;
               }
            case EUCLIDEANA:
               double suma=0;
                  for (int j=0;j<dim;j++)
                     suma+=M[j]*M[j];
               return sqrt(suma);
         }
      return -1;
   }

double Cvector::normalizar(int tipo)
   {
     double __norma;
      __norma=norma(tipo);
      if (__norma!=0)
         for (int i=0;i<dim;i++)
            M[i]/=__norma;
      return __norma;
   }

void Cvector::cero(void)
   {
      for (int i=0;i<dim;i++)
         M[i]=0;
   }

double Cvector::operator [](int k)
   {
      return M[k];
   }
double Cvector::operator *(Cvector &v)
   {
      double suma=0;
      for (int i=0;i<dim;i++)
         suma+=M[i]*v.M[i];
      return suma;
   }

Cvector& Cvector::operator *(double r)
   {
      Cvector *x;
      x=new Cvector(dim);
      for (int i=0;i<dim;i++)
         {            
            x->M[i]=r*M[i];
         }
      return *x;
   }

Cvector& Cvector::operator =(Cvector &v)
   {
      Cvector *x;
      x=new Cvector(dim);
      for (int i=0;i<dim;i++)
         {
            M[i]=v.M[i];
            x->M[i]=M[i];
         }
      return *x;
   }

Cvector& Cvector::operator +(Cvector &v)
   {
      Cvector *x;
      x=new Cvector(dim);
      if (v.dim!=dim)
         {
            warning=1;
            return *x;
         }

      for (int i=0;i<dim;i++)
        x->M[i]=v.M[i]+M[i];
      return *x;
   }

Cvector& Cvector::operator -(Cvector &v)
   {
      Cvector *x;
      x=new Cvector(dim);
      if (v.dim!=dim)
         {
            warning=1;
            return *x;
         }

      for (int i=0;i<dim;i++)
        x->M[i]=M[i]-v.M[i];
      return *x;
   }

Cvector& Cvector::operator +=(Cvector &v)
   {
      Cvector *x;
      x=new Cvector(dim);
      if (v.dim!=dim)
         {
            warning=1;
            return *x;
         }

      for (int i=0;i<dim;i++)
         {
           M[i]+=v.M[i];
           x->M[i]=M[i];
         }
      return *x;
   }

Cvector& Cvector::operator -=(Cvector &v)
   {
      Cvector *x;
      x=new Cvector(dim);
      if (v.dim!=dim)
         {
            warning=1;
            return *x;
         }

      for (int i=0;i<dim;i++)
         {
           M[i]-=v.M[i];
           x->M[i]=M[i];
         }
      return *x;
   }

Cvector& Cvector::operator *(CMatriz &T)
   {
      Cvector *aux;
      double suma;
      aux=new Cvector(T.col);
      if (dim!=T.fil)
         {
            aux->cero();
            return *aux;
         }
      for (int i=0;i<T.col;i++)
         {
            suma=0;
            for (int k=0;k<dim;k++)
               suma+=M[k]*T.M[k][i];
            aux->M[i]=suma;
         }
      return *aux;
   }


#endif //vectorH


